'use strict';

let createApplicationAsync = (() => {
  var _ref = _asyncToGenerator(function* (app) {
    let config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    try {
      if (!app) {
        app = express();
        config = require('./config');
        exports.config = config;
      }

      logger.initialize(config.logger);
      const server = http.createServer(app);

      var _ref2 = yield globals.initialize({ model: config.model });

      const raw = _ref2.raw,
            models = _ref2.models,
            services = _ref2.services,
            securitys = _ref2.securitys,
            middlewares = _ref2.middlewares;

      const api = yield blueprint.initialize(config.swagger, models);
      const io = createSocketIOApplication(server, config.io);

      app.io = io;
      exports.app = app;
      exports.io = io;
      exports.raw = raw;
      exports.models = models;
      exports.services = services;
      exports.securitys = securitys;
      exports.middlewares = exports.mws = middlewares;
      exports.api = api;
      exports.restMiddleware = function () {
        return getRestMiddleware(api, securitys, config);
      };

      if (rc.env === 'test') {
        exports.test = require('./supertest').initialize(app, api);
      }

      app.use(function (req, res, next) {
        req.state = {
          version: api.definition.info.version
        };
        const operation = api.getOperation(req);
        req.swagger = {
          operation: operation
        };
        next();
      });
      app.use(mws.session(config.session));
      app.listen = function listen() {
        return server.listen.apply(server, arguments);
      };

      return app;
    } catch (err) {
      console.log(err.stack); // eslint-disable-line
      process.exit(1);
    }
  });

  return function createApplicationAsync(_x) {
    return _ref.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const http = require('http');

const express = require('express');
const Router = require('router');

const rc = require('./rc');
const utility = require('./utility');
const Pipeline = require('./pipeline');
const mws = require('./middleware');
const logger = require('./logger');
const globals = require('./globals');
const blueprint = require('./blueprint');

var _require = require('./swagger');

const parseAsync = _require.parseAsync;

var _require2 = require('./error');

const HttpError = _require2.HttpError,
      ParameterError = _require2.ParameterError;

const createSocketIOApplication = require('./io');

exports.createApplicationAsync = createApplicationAsync;
// backward compability
exports.ResponseError = exports.HttpError = HttpError;
exports.ParameterError = ParameterError;
exports.translate = exports.utility = utility;
exports.parseAsync = parseAsync;
exports.logger = logger;
exports.rc = rc;
exports.wrapRoute = utility.wrapRoute;
exports.wrapMiddleware = utility.wrapMiddleware;

function getRestMiddleware(api, securitys, config) {
  const router = Router();

  //
  // Response pipeline
  //

  const pipeline = Pipeline();
  pipeline.use(mws.debug.postHandler());
  // remove response `null` fields
  pipeline.use(mws.validation.response());
  pipeline.use(mws.debug.response());
  router.use(pipeline.createMiddleware());

  //
  // Request pipeline
  //

  // binding request context
  router.use(mws.context(rc.target + '/context'));
  // setup swagger documentation
  router.use(mws.docs(api.definition));
  router.use(mws.switch());
  // request & response logging
  router.use(mws.debug.request());
  // request validation
  router.use(mws.validation.request());
  // security handler
  router.use(mws.security(api, securitys));
  // dynamic swagger defined route
  router.use(mws.debug.preHandler());
  router.use(api.basePathPrefix, mws.api(api));

  // 404
  router.use(api.basePathPrefix, () => {
    throw new HttpError('NotFound');
  });

  // error handling
  router.use(api.basePathPrefix, mws.errorHandler(api, config.errorHandler));

  return router;
}

process.on('unhandledRejection', function (reason, p) {
  let err = new Error('Unhandled Promise rejection');
  err.reason = reason;
  err.promise = p;
  logger.error('Unhandled Rejection at: Promise ', p, ' reason: ', reason);
  logger.error(reason.message, reason.stack);
  throw err;
});